@@ -79,6 +79,7 @@
 
 #include <gst/gst.h>
 #include <gst/gstvalue.h>
+#include <gst/video/navigation.h>
 #include <gst/video/video.h>
 #include <gst/riff/riff-ids.h>
 #include <string.h>
@@ -97,6 +98,8 @@ GST_DEBUG_CATEGORY_STATIC (clutter_gst_video_sink_debug);
   "I420,"                                       \
   "RGBA,"                                       \
   "BGRA,"                                       \
+  "RGBX,"                                       \
+  "BGRX,"                                       \
   "RGB,"                                        \
   "BGR,"                                        \
   "NV12 }"
@@ -116,18 +119,21 @@ static const char clutter_gst_video_sink_caps_str[] =
 ;
 
 
-  static GstStaticPadTemplate sinktemplate_all =
+static GstStaticPadTemplate sinktemplate_all =
     GST_STATIC_PAD_TEMPLATE ("sink",
                              GST_PAD_SINK,
                              GST_PAD_ALWAYS,
                              GST_STATIC_CAPS (clutter_gst_video_sink_caps_str));
 
 static void color_balance_iface_init (GstColorBalanceInterface *iface);
+static void navigation_interface_init (GstNavigationInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (ClutterGstVideoSink,
                          clutter_gst_video_sink,
                          GST_TYPE_VIDEO_SINK,
-                         G_IMPLEMENT_INTERFACE (GST_TYPE_COLOR_BALANCE, color_balance_iface_init))
+                         G_IMPLEMENT_INTERFACE (GST_TYPE_COLOR_BALANCE, color_balance_iface_init)
+                         G_IMPLEMENT_INTERFACE (GST_TYPE_NAVIGATION,
+                                                navigation_interface_init))
 
 enum
 {
@@ -678,6 +684,22 @@ color_balance_iface_init (GstColorBalanceInterface *iface)
 
 /**/
 
+static void
+clutter_gst_video_sink_navigation_send_event (GstNavigation *navigation,
+                                              GstStructure  *structure)
+{
+  // TODO: how do we feed the events back to the UI layer? New
+  // signals?
+}
+
+static void
+navigation_interface_init (GstNavigationInterface * iface)
+{
+  iface->send_event = clutter_gst_video_sink_navigation_send_event;
+}
+
+/**/
+
 static void
 clutter_gst_source_finalize (GSource *source)
 {
@@ -971,16 +993,6 @@ dirty_default_pipeline (ClutterGstVideoSink *sink)
     }
 }
 
-static int
-_clutter_gst_video_sink_get_video_layer (ClutterGstVideoSink *sink)
-{
-  ClutterGstVideoSinkPrivate *priv = sink->priv;
-
-  if (clutter_gst_video_sink_needs_color_balance_shader (sink))
-    return priv->custom_start + 3;
-  return priv->custom_start;
-}
-
 static void
 clear_frame_textures (ClutterGstVideoSink *sink)
 {
@@ -1344,7 +1356,7 @@ static ClutterGstRenderer rgb32_glsl_renderer =
                      ";"
 #endif
                      MAKE_CAPS (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
-                                "{ RGBA, BGRA }")),
+                                "{ RGBA, BGRA, RGBx, BGRx }")),
     1, /* n_layers */
     clutter_gst_rgb32_glsl_setup_pipeline,
     clutter_gst_rgb32_upload,
@@ -1358,7 +1370,7 @@ static ClutterGstRenderer rgb32_renderer =
     CLUTTER_GST_RGB32,
     0,
     GST_STATIC_CAPS (MAKE_CAPS (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
-                                "{ RGBA, BGRA }")),
+                                "{ RGBA, BGRA, RGBx, BGRx }")),
     2, /* n_layers */
     clutter_gst_rgb32_setup_pipeline,
     clutter_gst_rgb32_upload,
@@ -1802,8 +1814,9 @@ clutter_gst_video_sink_get_caps (GstBaseSink *bsink,
   if (sink->priv->caps == NULL)
     return NULL;
 
-  /* if (filter != NULL) */
-  /*   return gst_caps_intersect (sink->priv->caps, filter); */
+  if (filter != NULL)
+    return gst_caps_intersect_full (filter, sink->priv->caps,
+                                    GST_CAPS_INTERSECT_FIRST);
   return gst_caps_ref (sink->priv->caps);
 }
 
@@ -1852,10 +1865,12 @@ clutter_gst_video_sink_parse_caps (GstCaps *caps,
       bgr = TRUE;
       break;
     case GST_VIDEO_FORMAT_RGBA:
+    case GST_VIDEO_FORMAT_RGBx:
       format = CLUTTER_GST_RGB32;
       bgr = FALSE;
       break;
     case GST_VIDEO_FORMAT_BGRA:
+    case GST_VIDEO_FORMAT_BGRx:
       format = CLUTTER_GST_RGB32;
       bgr = TRUE;
       break;
